/* Programme test de la liaison I2C entre la carte Raspberry et la carte Arduino.
 * Auteur:
 * Date:
 *  Le Raspberry est maitre de la communication et l'arduino esclave.
 *  Utilisation de la librairie Wire:
 *  https://www.arduino.cc/en/Reference/Wire
 *  http://www.mon-club-elec.fr/pmwiki_reference_arduino/pmwiki.php?n=Main.LibrairieWire
 *  Programme minimum pour déclarer le bus i2c: 
 *    L'utilisation de la librairie "Wire"
 *    L'adresse de la carte Arduino est déclaré en hexadécimale..
 */
 
#include <Wire.h>

#define SLAVE_ADDRESS 0x10// Déclaration de l'adresse de la carte Arduino

int valA0 = 0;   // valeur de conversion de l'entrée A0 sur 16 bits
byte valA0h = 0; // Octet de poids fort de valA0
byte valA0l = 0; // Octet de poids faible de valA0
int tabcap[2] = {valA0h,valA0l}; //tableau de 2 valeurs
int i=0; //index du tableau
const byte Led = 13; // Sortie logique commandée par la carte Raspberry
//byte commandeLed = 0;
int dataReceived = 0;

// Initialisation
void setup() {
    Wire.begin(SLAVE_ADDRESS); // Initialisation de la liaison i2c
	  Wire.onRequest(sendData); // init sous-programme émission  sur le bus i2c
    Wire.onReceive(receiveData); // initialisation sous-programme réception sur le bus i2c
    Serial.begin(9600); // Initialisation vitesse de communication de la liaison serie (USB) 9600 baud
    pinMode(Led, OUTPUT); // La pin N°13 est déclarée en sortie pour piloter la Led
}

// Programme principal
void loop() {
    valA0 = analogRead(A0);         // Conversion de l'entree A0, converti en un integer sur 16 bits (int val=).
    Serial.println (valA0);         // Ecriture de la valeur "val" sur le terminal (via la liaison serie).
    valA0h = (char)highByte(valA0); // Isolation de l'octet de poids fort.
    tabcap[0] = valA0h;             // Stockage dans le tableau, attention [2] correspond au troisieme emplacement du tableau, il commence a [0].
    valA0l = (char)lowByte(valA0);  // Isolation de l'octet de poids faible.
    tabcap[1] = valA0l;             // Stockage dans le tableau, attention [3].
    digitalWrite(Led, dataReceived);
    delay(20);
     
}

// Sous programme emission bus i2c
void sendData(){
    int envoi = tabcap[i];        // envoi contient l'octet à transmettre
    if (i==0) {                   // écrire dans la console Arduino l'octet transmis
      Serial.print("valA0h = ");  // son nom
    }
     if (i==1) {
      Serial.print("valA0l = ");  // son nom
    }
    Serial.println(tabcap[i]);    // sa valeur
    Wire.write(envoi);            // emission de l'octet sur le bus
    i=i+1;                        // pointe l'octet suivant
    if (i==2) {                   // initialiser le pointeur
      i=0;                        // si on dépasse le tableau
    }
}

// Sous programme Réception bus i2c
void receiveData(int byteCount){
    while(Wire.available()) {
        dataReceived = Wire.read();
        Serial.print("Donnee recue : ");
        Serial.println(dataReceived);
    }
}                    
